using System;
using System.Drawing;
using System.Collections;
using System.Configuration;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Data.SqlClient;
using System.IO;

using Syncfusion.Windows.Forms.Chart;
using Syncfusion.Drawing;

namespace chart
{
	public struct PRGData
	{
		DateTime date;
		int hour;
		double p_qty;
		double m_qty;

		public PRGData(DateTime date, int hour, double p_qty, double m_qty)
		{
			this.date = date;
			this.hour = hour;
			this.p_qty = p_qty;
			this.m_qty = m_qty;
		}

		public DateTime Date
		{
			get { return date; }
		}

		public int Hour
		{
			get { return hour; }
		}

		public double P_Qty
		{
			get { return p_qty; }
		}

		public double M_Qty
		{
			get { return m_qty; }
		}
	}

	public enum TypeInterval
	{
		Date,
		Week,
		Month,
		Year
	}

	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class MainForm : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private System.Windows.Forms.ComboBox cbDate;
		private Syncfusion.Windows.Forms.Chart.ChartControl myChart;
		private ChartAxis secYAxis = null;
		private PRGData[] myData;
		private TypeInterval type;
		private DateTime startDate, endDate, startPeriod, endPeriod;
		private int tol_up, tol_down;
		private int countAll, countPeriod, inc = 0;
		private double Max_p, Min_p, Int_p, Max_diff, Min_diff, Int_diff;
		private System.Windows.Forms.CheckBox cbMonth;

		public MainForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
            
			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			startDate = Convert.ToDateTime("01/01/2006");
			endDate = Convert.ToDateTime("01/31/2006");
			countAll = 744;//24*31 januar
			tol_up = tol_down = 10;
			type = TypeInterval.Month;
			myData = new PRGData[countAll];

			try
			{
				string data = ConfigurationSettings.AppSettings["dataDirectory"];
				string[] fields = null;
				using (StreamReader sr = new StreamReader(data)) 
				{
					String line;
					int j = 0;
					char[] odd = new char[1];
					odd[0] = '	';
					while ((line = sr.ReadLine()) != null) 
					{
						fields = line.Split(odd);
						myData[j] = new PRGData(Convert.ToDateTime(fields[0]), Convert.ToInt16(fields[1]), Convert.ToDouble(fields[2]), Convert.ToDouble(fields[3]));
						j++;
					}
				}

			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
			}
			
			chart_appearance();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.myChart = new Syncfusion.Windows.Forms.Chart.ChartControl();
			this.cbDate = new System.Windows.Forms.ComboBox();
			this.cbMonth = new System.Windows.Forms.CheckBox();
			this.SuspendLayout();
			// 
			// myChart
			// 
			this.myChart.BackColor = System.Drawing.Color.LightSteelBlue;
			// 
			// myChart.ChartArea
			// 
			this.myChart.ChartArea.BackColor = System.Drawing.Color.Gainsboro;
			this.myChart.ChartArea.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.myChart.ChartArea.TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit;
			this.myChart.ChartInterior = new Syncfusion.Drawing.BrushInfo(Syncfusion.Drawing.GradientStyle.None, System.Drawing.Color.White, System.Drawing.SystemColors.Window);
			this.myChart.Controls.Add(this.myChart.Legend);
			this.myChart.Dock = System.Windows.Forms.DockStyle.Fill;
			this.myChart.Font = new System.Drawing.Font("Arial", 12F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			// 
			// myChart.Legend
			// 
			this.myChart.Legend.Location = new System.Drawing.Point(368, 39);
			this.myChart.Legend.Name = "";
			this.myChart.Legend.Orientation = Syncfusion.Windows.Forms.Chart.ChartOrientation.Horizontal;
			this.myChart.Legend.TabIndex = 3;
			this.myChart.LegendPosition = Syncfusion.Windows.Forms.Chart.ChartDock.Top;
			this.myChart.Location = new System.Drawing.Point(0, 0);
			this.myChart.Name = "myChart";
			this.myChart.PrimaryXAxis.DrawGrid = false;
			this.myChart.PrimaryXAxis.Orientation = Syncfusion.Windows.Forms.Chart.ChartOrientation.Horizontal;
			this.myChart.PrimaryXAxis.Range = new Syncfusion.Windows.Forms.Chart.MinMaxInfo(0, 25, 1);
			this.myChart.PrimaryXAxis.RangeType = Syncfusion.Windows.Forms.Chart.ChartAxisRangeType.Set;
			this.myChart.PrimaryXAxis.Title = "Time (Hour)";
			this.myChart.PrimaryYAxis.DrawGrid = false;
			this.myChart.PrimaryYAxis.Range = new Syncfusion.Windows.Forms.Chart.MinMaxInfo(0, 0, 0);
			this.myChart.PrimaryYAxis.RangeType = Syncfusion.Windows.Forms.Chart.ChartAxisRangeType.Set;
			this.myChart.PrimaryYAxis.Title = "Consumption (kWh)";
			this.myChart.ShowToolTips = true;
			this.myChart.Size = new System.Drawing.Size(812, 605);
			this.myChart.TabIndex = 0;
			this.myChart.Text = "Chart Example";
			this.myChart.TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit;
			this.myChart.ToolBar.Location = new System.Drawing.Point(0, 0);
			this.myChart.ChartFormatAxisLabel += new Syncfusion.Windows.Forms.Chart.ChartFormatAxisLabelEventHandler(this.myChart_ChartFormatAxisLabel);
			// 
			// cbDate
			// 
			this.cbDate.Items.AddRange(new object[] {
														"01/01/2006",
														"01/02/2006",
														"01/03/2006",
														"01/04/2006",
														"01/05/2006",
														"01/06/2006",
														"01/07/2006",
														"01/08/2006",
														"01/09/2006",
														"01/10/2006",
														"01/11/2006",
														"01/12/2006",
														"01/13/2006",
														"01/14/2006",
														"01/15/2006",
														"01/16/2006",
														"01/17/2006",
														"01/18/2006",
														"01/19/2006",
														"01/20/2006",
														"01/21/2006",
														"01/22/2006",
														"01/3/2006",
														"01/24/2006",
														"01/25/2006",
														"01/26/2006",
														"01/27/2006",
														"01/28/2006",
														"01/29/2006",
														"01/30/2006",
														"01/31/2006"});
			this.cbDate.Location = new System.Drawing.Point(48, 40);
			this.cbDate.Name = "cbDate";
			this.cbDate.Size = new System.Drawing.Size(121, 21);
			this.cbDate.TabIndex = 1;
			this.cbDate.Text = "cbDate";
			this.cbDate.SelectedIndexChanged += new System.EventHandler(this.cbDate_SelectedIndexChanged);
			// 
			// cbMonth
			// 
			this.cbMonth.BackColor = System.Drawing.Color.LightSteelBlue;
			this.cbMonth.Location = new System.Drawing.Point(188, 40);
			this.cbMonth.Name = "cbMonth";
			this.cbMonth.TabIndex = 2;
			this.cbMonth.Text = "Month";
			this.cbMonth.CheckedChanged += new System.EventHandler(this.cbMonth_CheckedChanged);
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(812, 605);
			this.Controls.Add(this.cbMonth);
			this.Controls.Add(this.cbDate);
			this.Controls.Add(this.myChart);
			this.Name = "MainForm";
			this.Text = "Main";
			this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
			this.Load += new System.EventHandler(this.MainForm_Load);
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new MainForm());
		}


		private void chart_appearance()
		{// We don't use indexed mode with multiple axes.
			myChart.Indexed = false;
			myChart.LegendPosition = ChartDock.Top;
			myChart.Legend.Visible = true;
			myChart.HidePartialLabels = true;
			myChart.ShowToolTips = true;
			// add the second axis
			if (secYAxis == null)
			{
				secYAxis = new ChartAxis();
				myChart.Axes.Add(this.secYAxis);
				secYAxis.OpposedPosition = true; 
				secYAxis.Orientation = ChartOrientation.Vertical;
				secYAxis.DrawGrid = false;
				secYAxis.RangeType = ChartAxisRangeType.Set;
				//secYAxis.Range = new MinMaxInfo(0,5,1);
				secYAxis.Title = "Difference OUT (kWh)";
				secYAxis.TitleAlignment = StringAlignment.Center;
			}
			// Specify primary axis
			myChart.PrimaryXAxis.DrawGrid = false;
			myChart.PrimaryXAxis.RangeType = ChartAxisRangeType.Set;
			//myChart.PrimaryXAxis.Range = new MinMaxInfo(0, 0, 0);
			myChart.PrimaryXAxis.Title = "Time (Hour / Date)";
			myChart.PrimaryXAxis.TitleAlignment = StringAlignment.Center;

			//The Tick Size is 0 here.
			myChart.PrimaryXAxis.TickSize = new Size(0,0);
			
			myChart.PrimaryYAxis.DrawGrid = false;
			myChart.PrimaryYAxis.RangeType = ChartAxisRangeType.Set;
			//myChart.PrimaryYAxis.Range = new MinMaxInfo(0, 5, 1);
			myChart.PrimaryYAxis.Title = "Consumption (kWh)";
			myChart.PrimaryYAxis.TitleAlignment = StringAlignment.Center;
				
			
			// Specify a ChartPalette:
			Color[] color = new Color[]{Color.FromArgb(255,Color.Red), Color.FromArgb(0, Color.White), Color.FromArgb(150, Color.Red),Color.FromArgb(255,Color.Blue)};
			myChart.Palette = ChartColorPalette.Custom ;
			myChart.Model.ColorModel.CustomColors = color;
			myChart.Palette = ChartColorPalette.Custom ;

			myChart.ChartFormatAxisLabel += new ChartFormatAxisLabelEventHandler(myChart_ChartFormatAxisLabel);
			myChart.Legend.FilterItems += new LegendFilterItemsEventHandler(Legend_FilterItems);
		}

	
		private void chart_data(DateTime startD, DateTime endD)
		{
			myChart.Series.Clear();
			myChart.BeginUpdate();
			if (countAll > 0)
			{
				startPeriod = startD;
				endPeriod = endD;

				if (startD == endD)
				{
					countPeriod = 24;
					type = TypeInterval.Date;
				}
				else								
				{
					countPeriod = 744; //31*24
					type = TypeInterval.Month;
				}
				// Odchylka
				ChartSeries seriesDiff = myChart.Model.NewSeries("Series_diff");
				
				seriesDiff.Text = "Difference OUT";
				seriesDiff.Type = ChartSeriesType.StackingColumn;
				seriesDiff.Style.Border.Color = Color.FromArgb(255,Color.Red);
				seriesDiff.Style.Font.Bold = true;
				if ((type == TypeInterval.Date) || (type == TypeInterval.Week))
				{
					seriesDiff.Style.Symbol.Shape = ChartSymbolShape.Circle;
					seriesDiff.Style.Symbol.Color = Color.Black;
					//The symbol size is set to a greater value
					//seriesDiff.Style.Symbol.Size = new Size(2,2);

					seriesDiff.Style.Symbol.Size = new Size(5,5);
				}
	
				// Prediction - tol , + tol
				ChartSeries seriesPG1 = myChart.Model.NewSeries("Series_pred-", ChartSeriesType.StackingArea);
				ChartSeries seriesPG2 = myChart.Model.NewSeries("Series_pred+", ChartSeriesType.StackingArea);

				seriesPG2.Text = "Tolerance interval";
				ChartStyleInfo style1 =  seriesPG1.Style;
				ChartStyleInfo style2 = seriesPG2.Style;
		
				style1.Border.Color = Color.FromArgb(0,Color.Red);
				style1.Font.Bold = true;
				if ((type == TypeInterval.Date) || (type == TypeInterval.Week))
				{
					style1.Symbol.Shape = ChartSymbolShape.Circle;
					style1.Symbol.Color = Color.Black;
					style1.Symbol.Size = new Size(2,2);
				}
				style1.DisplayShadow = false;
					
				style2.Border.Color = Color.FromArgb(150,Color.Red);
				style2.Font.Bold = true;
				if ((type == TypeInterval.Date) || (type == TypeInterval.Week))
				{
					style2.Symbol.Shape = ChartSymbolShape.Circle;
					style2.Symbol.Color = Color.Black;
					style2.Symbol.Size = new Size(2,2);
				}
				style2.DisplayShadow = false;

				// Measured
				ChartSeries seriesMPG = myChart.Model.NewSeries("Series_meas");
			
				seriesMPG.Text = "Measured";
				seriesMPG.Type = ChartSeriesType.Line;
				seriesMPG.Style.Font.Bold = true;
				if ((type == TypeInterval.Date) || (type == TypeInterval.Week))
				{
					seriesMPG.Style.Symbol.Shape = ChartSymbolShape.Circle;
					seriesMPG.Style.Symbol.Color = Color.Black;
					seriesMPG.Style.Symbol.Size = new Size(2,2);
				}
				seriesMPG.Style.DisplayShadow = false;
		
				// points for chart
				DateTime date;
				date = myData[0].Date;
				int i = 0, startPos, endPos, pos = 1;
				double vysledok, up, down, data;
				while ((i < countAll) && (myData[i].Date != startD.Date)) {i++;}
				startPos = i;
				//		seriesDiff.Points.Add(0,0);
				while ((i < countAll) && (myData[i].Date != endD.AddDays(1).Date))
				{
					vysledok = 0;
					up = Math.Round(myData[i].P_Qty * (1 + tol_up / 100.00), 3);
					down = Math.Round(myData[i].P_Qty * (1 - tol_down / 100.00), 3);
					data = myData[i].M_Qty;
					if (data > up) vysledok = Math.Abs(data - up);
					if (data < down ) vysledok = Math.Abs(down - data);
					seriesDiff.Points.Add(pos, vysledok);
					seriesPG1.Points.Add(pos, down - vysledok);
					seriesPG2.Points.Add(pos, up - down);
					seriesMPG.Points.Add(pos, data);// line ok
					i++;
					pos++;
				}
				endPos = i;

				
				seriesDiff.YAxis = this.secYAxis;

				seriesDiff.PrepareStyle +=new ChartPrepareStyleInfoHandler(this.ChartControlSeries_PrepareStyle);			
				seriesPG1.PrepareStyle +=new ChartPrepareStyleInfoHandler(this.ChartControlSeries_PrepareStyle);
				seriesPG2.PrepareStyle +=new ChartPrepareStyleInfoHandler(this.ChartControlSeries_PrepareStyle);
				seriesMPG.PrepareStyle +=new ChartPrepareStyleInfoHandler(this.ChartControlSeries_PrepareStyle);			
				
				myChart.Series.Add(seriesDiff);
				myChart.Series.Add(seriesPG1);
				myChart.Series.Add(seriesPG2);
				myChart.Series.Add(seriesMPG);
				
				getRange(startPos, endPos);
				change_range();					
				myChart.EndUpdate();
			}
		}


		protected void ChartControlSeries_PrepareStyle(object sender, ChartPrepareStyleInfoEventArgs args)
		{
			// Style formatting using a callback. You can apply the same settings directly on the series style on the
			// point styles.
			ChartSeries series = sender as ChartSeries;
			if(series != null)
			{
				double kwh = series.Points[args.Index].YValues[0];
				if (series.Name == "Series_pred+")
				{
					kwh = kwh + myChart.Series["Series_pred-"].Points[args.Index].YValues[0]
						+ myChart.Series["Series_diff"].Points[args.Index].YValues[0];;
				}
				if (series.Name == "Series_pred-")
				{
					kwh = kwh + myChart.Series["Series_diff"].Points[args.Index].YValues[0];;
				}
				double h = (series.Points[args.Index].X - 1) % 24;
				string xValue = h.ToString("0")+":00 - " + (h+1).ToString("0")+":00 h";
				string yValue;
				if ((kwh % 1) == 0) yValue = kwh.ToString("0");
				else yValue = kwh.ToString("0.00");
				args.Style.ToolTip = yValue+ " kWh\n"+ xValue;
				args.Handled = true;
			}
		}

		private void MainForm_Load(object sender, System.EventArgs e)
		{	
			cbMonth.Checked = true;
		}

		private void myChart_ChartFormatAxisLabel(object sender, Syncfusion.Windows.Forms.Chart.ChartFormatAxisLabelEventArgs e)
		{
			if (e.AxisOrientation == ChartOrientation.Vertical)
			{
				if (!e.IsAxisPrimary)
				{
					e.Label = e.Value.ToString("0.000");
					if (Int_diff != 0)
					{
						if (e.Value > Max_diff + Int_diff)
							e.Label = "";
					}
					else if (e.Value > 5) e.Label = "";
				}
				else
				{
					if (e.Value < (Min_p - Int_p)) e.Label = "";
					else

						if (myChart.PrimaryYAxis.Range.Interval < 1.0)
						{
							e.Label = e.Value.ToString("0.00");
						}
						else
						{
							e.Label = e.Value.ToString("0.000");
						}
				}
				e.Handled = true;
			}

			if (e.IsAxisPrimary && e.AxisOrientation == ChartOrientation.Horizontal)
			{
				if (type == TypeInterval.Month) 
				{
					e.Label = "";
					
					if (e.Value == 0) inc = 0; 
					else inc++;
					if ((inc % 24) == 12) 
						e.Label = startPeriod.AddDays(inc / 24).ToString("d MMM") + "  ";
				}

				
				if (e.Value == (countPeriod + 1))
				{
					e.Label = "";
				};
				e.Handled = true;
				if (countPeriod == 0) e.Label = e.Value.ToString("0")  + "  ";
			}
		}

		private void cbDate_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			chart_data(Convert.ToDateTime(cbDate.Text), Convert.ToDateTime(cbDate.Text));
			cbMonth.Checked = false;
		}

		private void getRange(int start, int end)
		{
			Max_p = 0;
			Min_p = 1000000;
			Max_diff = 0;
			Min_diff = 1000000;
			double down, up, diff;
			for (int i=start; i<end; i++)
			{
				up = Math.Round(myData[i].P_Qty * (1 + tol_up / 100.00), 3);
				down = Math.Round(myData[i].P_Qty * (1 - tol_down / 100.00), 3);
				Max_p = Math.Max(Max_p, Math.Max(up, myData[i].M_Qty));
				Min_p = Math.Min(Min_p , Math.Min(down, myData[i].M_Qty));
				diff = 0;
				if (myData[i].M_Qty > up) diff = myData[i].M_Qty - up;
				if (myData[i].M_Qty < down) diff = down - myData[i].M_Qty;
				Max_diff = Math.Max(Max_diff, diff);
				Min_diff = Math.Min(Min_diff, diff);
			}
			//ranges
			/*
			Int_p = (max_p - min_p) / 5.00;
			Max_p = max_p + Int_p / 2.00;
			Min_p = min_p - (Int_p * 6);

			Int_diff = (max_diff / 5.00);//Math.Round((max_diff - min_diff)/ 5, 0);
			Max_diff = Int_diff * 11;//max_diff + Int_diff * 6;//Int_diff*10 + Int_diff/2;
			Min_diff = 0;//min_diff - Int_diff/2;
			*/
		}
	
		private void change_range()
		{
			// primary X Axis
			myChart.PrimaryXAxis.Range.Min = 0;
			myChart.PrimaryXAxis.Range.Max = countPeriod + 1;

			//This is what I changed
			//myChart.PrimaryXAxis.Range.Interval = 1;
			if(type == TypeInterval.Month)
				myChart.PrimaryXAxis.Range.Interval = 2;
			else
				myChart.PrimaryXAxis.Range.Interval = 1;
			

			if ((type != TypeInterval.Date) || (type != TypeInterval.Year))
			{
				myChart.PrimaryXAxis.LabelRotate = true;
				myChart.PrimaryXAxis.LabelRotateAngle = 270;
			}

			Int_p = (Max_p - Min_p) / 5.00;
			Int_diff = (Max_diff / 5.00);//Math.Round((max_diff - min_diff)/ 5, 0);
			
			// Y Axis
			myChart.PrimaryYAxis.Range.Interval = Int_p;
			myChart.PrimaryYAxis.Range.Max = Max_p + Int_p / 2.00;
			myChart.PrimaryYAxis.Range.Min = Min_p - (Int_p * 6);

			//Changed here also
			if (Int_diff != 0)
			{
//				secYAxis.Range.Min = 0;//Min_diff - Int_diff/2;
//				secYAxis.Range.Max = Int_diff*11;// + Int_diff/2;//Max_diff + Int_diff * 6;
//				secYAxis.Range.Interval = Int_diff;
				double c= (Max_diff / 5)*11;
				double rem = c%100;
				secYAxis.Range.Min = 0;
				secYAxis.Range.Interval = Max_diff / 5;
				secYAxis.Range.Max = c + (100 - rem);

			}
			else secYAxis.Range = new MinMaxInfo(-1, 12, 1);
		}


		private void Legend_FilterItems(object sender, ChartLegendFilterItemsEventArgs e)
		{
			
			if (e.Items.Count == 4)
			{
				int legendCount = e.Items.Count;
				ChartLegendItemsCollection filteredItems = new ChartLegendItemsCollection();
				int j = 0;
				for(int i =0; i <legendCount; i++ )
				{
					if(i != 1)
					{
						filteredItems.Add(e.Items[i]);
						j++;
					}
				}
				e.Items = filteredItems;
			}
		}

		private void cbMonth_CheckedChanged(object sender, System.EventArgs e)
		{
			if (cbMonth.Checked)
			{
				chart_data(Convert.ToDateTime("01/01/2006"), Convert.ToDateTime("01/31/2006"));
			}
		}
		
	}
}

